package entity_crud

import (
	"git.sxidc.com/go-framework/baize/framework/core/api/response"
	"git.sxidc.com/go-framework/baize/framework/gateway"
	"git.sxidc.com/go-tools/utils/http_client"
	"git.sxidc.com/go-tools/utils/template"
	"github.com/iancoleman/strcase"
	"net/http"
)

func BindSimple(builder *gateway.Builder, params *Simple, opts ...any) {
	globalOptions := new(GlobalOptions)
	createOptions := new(CreateOptions)
	deleteOptions := new(DeleteOptions)
	updateOptions := new(UpdateOptions)
	queryOptions := new(QueryOptions)
	getByIDOptions := new(GetByIDOptions)

	for _, opt := range opts {
		switch o := opt.(type) {
		case GlobalOption:
			o(globalOptions)
		case CreateOption:
			o(createOptions)
		case DeleteOption:
			o(deleteOptions)
		case UpdateOption:
			o(updateOptions)
		case QueryOption:
			o(queryOptions)
		case GetByIDOption:
			o(getByIDOptions)
		default:
			continue
		}
	}

	params.globalOptions = globalOptions
	params.createOptions = createOptions
	params.deleteOptions = deleteOptions
	params.updateOptions = updateOptions
	params.queryOptions = queryOptions
	params.getByIDOptions = getByIDOptions

	params.bind(builder)
}

type Simple struct {
	// 除去后缀的服务URL，如http://localhost:8080/example/api/v1
	ServiceVersionedUrl string

	// 服务缩写
	ServiceShortName string

	// 领域名称
	DomainCamelName string

	// 可选配置项，通过WithXXX配置
	globalOptions  *GlobalOptions
	createOptions  *CreateOptions
	deleteOptions  *DeleteOptions
	updateOptions  *UpdateOptions
	queryOptions   *QueryOptions
	getByIDOptions *GetByIDOptions
}

func (simple *Simple) bind(builder *gateway.Builder) {
	globalOptions := simple.globalOptions
	createOptions := simple.createOptions
	deleteOptions := simple.deleteOptions
	updateOptions := simple.updateOptions
	queryOptions := simple.queryOptions
	getByIDOptions := simple.getByIDOptions

	domainPath := "/" + strcase.ToLowerCamel(template.Id(simple.DomainCamelName))

	// 创建
	if !createOptions.disable {
		createMiddlewares := append(globalOptions.middlewares, createOptions.middlewares...)

		builder.PostRouteWithTenantIDAndCreateUserInfo("/"+simple.ServiceShortName+domainPath+"/create",
			func(requestBuilder *gateway.RequestBuilder) {
				if createOptions.responseSuccessCallback != nil {
					requestBuilder.ResponseSuccessCallback(createOptions.responseSuccessCallback)
				}

				requestBuilder.
					Post(
						&gateway.PostRequest{
							Url: simple.ServiceVersionedUrl + domainPath + "/create",
						},
						gateway.WithBeforeRequestCallback(
							func(requestBuilder *gateway.RequestBuilder) error {
								if createOptions.beforeBuilderCallback != nil {
									return createOptions.beforeBuilderCallback(requestBuilder)
								}

								return nil
							}),
						gateway.WithRequestResponseCallback(
							func(requestBuilder *gateway.RequestBuilder, response *http_client.Response) error {
								if createOptions.afterBuilderCallback != nil {
									err := createOptions.afterBuilderCallback(requestBuilder)
									if err != nil {
										return err
									}
								}

								return nil
							})).
					ResponseErrorCallback(func(requestBuilder *gateway.RequestBuilder, err error) {
						response.SendIDResponse(requestBuilder.Context(), http.StatusOK, "", err)
					}).
					Request()
			}, createMiddlewares...)
	}

	// 删除
	if !deleteOptions.disable {
		deleteMiddlewares := append(globalOptions.middlewares, deleteOptions.middlewares...)

		builder.DeleteRouteWithDeleteUserInfo("/"+simple.ServiceShortName+domainPath+"/delete",
			func(requestBuilder *gateway.RequestBuilder) {
				if deleteOptions.responseSuccessCallback != nil {
					requestBuilder.ResponseSuccessCallback(deleteOptions.responseSuccessCallback)
				}

				requestBuilder.
					Delete(
						&gateway.DeleteRequest{
							Url: simple.ServiceVersionedUrl + domainPath + "/delete",
						},
						gateway.WithBeforeRequestCallback(
							func(requestBuilder *gateway.RequestBuilder) error {
								if deleteOptions.beforeBuilderCallback != nil {
									err := deleteOptions.beforeBuilderCallback(requestBuilder)
									if err != nil {
										return err
									}
								}

								return nil
							}),
						gateway.WithRequestResponseCallback(
							func(requestBuilder *gateway.RequestBuilder, response *http_client.Response) error {
								if deleteOptions.afterBuilderCallback != nil {
									err := deleteOptions.afterBuilderCallback(requestBuilder)
									if err != nil {
										return err
									}
								}

								return nil
							})).
					Request()
			}, deleteMiddlewares...)
	}

	// 修改
	if !updateOptions.disable {
		updateMiddlewares := append(globalOptions.middlewares, updateOptions.middlewares...)

		builder.PutRouteWithUpdateUserInfo("/"+simple.ServiceShortName+domainPath+"/update",
			func(requestBuilder *gateway.RequestBuilder) {
				if updateOptions.responseSuccessCallback != nil {
					requestBuilder.ResponseSuccessCallback(updateOptions.responseSuccessCallback)
				}

				requestBuilder.
					Put(
						&gateway.PutRequest{
							Url: simple.ServiceVersionedUrl + domainPath + "/update",
						},
						gateway.WithBeforeRequestCallback(func(requestBuilder *gateway.RequestBuilder) error {
							if updateOptions.beforeBuilderCallback != nil {
								err := updateOptions.beforeBuilderCallback(requestBuilder)
								if err != nil {
									return err
								}
							}

							return nil
						}),
						gateway.WithRequestResponseCallback(
							func(requestBuilder *gateway.RequestBuilder, response *http_client.Response) error {
								if updateOptions.afterBuilderCallback != nil {
									err := updateOptions.afterBuilderCallback(requestBuilder)
									if err != nil {
										return err
									}
								}

								return nil
							})).
					Request()
			}, updateMiddlewares...)
	}

	// 查询
	if !queryOptions.disable {
		queryMiddlewares := append(globalOptions.middlewares, queryOptions.middlewares...)

		builder.GetRouteWithTenantID("/"+simple.ServiceShortName+domainPath+"/query",
			func(requestBuilder *gateway.RequestBuilder) {
				if queryOptions.responseSuccessCallback != nil {
					requestBuilder.ResponseSuccessCallback(queryOptions.responseSuccessCallback)
				}

				requestBuilder.
					Get(
						&gateway.GetRequest{
							Url: simple.ServiceVersionedUrl + domainPath + "/query",
						},
						gateway.WithBeforeRequestCallback(func(requestBuilder *gateway.RequestBuilder) error {
							if queryOptions.beforeBuilderCallback != nil {
								err := queryOptions.beforeBuilderCallback(requestBuilder)
								if err != nil {
									return err
								}
							}

							return nil
						}),
						gateway.WithRequestResponseCallback(
							func(requestBuilder *gateway.RequestBuilder, response *http_client.Response) error {
								if queryOptions.afterBuilderCallback != nil {
									err := queryOptions.afterBuilderCallback(requestBuilder)
									if err != nil {
										return err
									}
								}

								return nil
							})).
					Request()
			}, queryMiddlewares...)
	}

	// 通过ID获取
	if !getByIDOptions.disable {
		getByIDMiddlewares := append(globalOptions.middlewares, getByIDOptions.middlewares...)

		builder.GetRoute("/"+simple.ServiceShortName+domainPath+"/get",
			func(requestBuilder *gateway.RequestBuilder) {
				if getByIDOptions.responseSuccessCallback != nil {
					requestBuilder.ResponseSuccessCallback(getByIDOptions.responseSuccessCallback)
				}

				requestBuilder.
					Get(
						&gateway.GetRequest{
							Url: simple.ServiceVersionedUrl + domainPath + "/get",
						},
						gateway.WithBeforeRequestCallback(
							func(requestBuilder *gateway.RequestBuilder) error {
								if getByIDOptions.beforeBuilderCallback != nil {
									err := getByIDOptions.beforeBuilderCallback(requestBuilder)
									if err != nil {
										return err
									}
								}

								return nil
							}),
						gateway.WithRequestResponseCallback(
							func(requestBuilder *gateway.RequestBuilder, response *http_client.Response) error {
								if getByIDOptions.afterBuilderCallback != nil {
									err := getByIDOptions.afterBuilderCallback(requestBuilder)
									if err != nil {
										return err
									}
								}

								return nil
							})).
					Request()
			}, getByIDMiddlewares...)
	}
}

type RequestBuilderCallback func(requestBuilder *gateway.RequestBuilder) error

type GlobalOption func(options *GlobalOptions)
type CreateOption func(options *CreateOptions)
type DeleteOption func(options *DeleteOptions)
type UpdateOption func(options *UpdateOptions)
type QueryOption func(options *QueryOptions)
type GetByIDOption func(options *GetByIDOptions)

type GlobalOptions struct {
	middlewares []gateway.Handler
}

type CreateOptions struct {
	// 关闭创建
	disable bool

	// 创建请求前回调
	beforeBuilderCallback RequestBuilderCallback

	// 创建请求后回调
	afterBuilderCallback RequestBuilderCallback

	// responseSuccessCallback 成功响应回调
	responseSuccessCallback gateway.ResponseSuccessCallback

	// 创建中间件
	middlewares []gateway.Handler
}

type DeleteOptions struct {
	// 关闭删除
	disable bool

	// 删除请求前回调
	beforeBuilderCallback RequestBuilderCallback

	// 删除请求后回调
	afterBuilderCallback RequestBuilderCallback

	// responseSuccessCallback 成功响应回调
	responseSuccessCallback gateway.ResponseSuccessCallback

	// 删除中间件
	middlewares []gateway.Handler
}

type UpdateOptions struct {
	// 关闭更新
	disable bool

	// 更新请求前回调
	beforeBuilderCallback RequestBuilderCallback

	// 更新请求后回调
	afterBuilderCallback RequestBuilderCallback

	// responseSuccessCallback 成功响应回调
	responseSuccessCallback gateway.ResponseSuccessCallback

	// 更新中间件
	middlewares []gateway.Handler
}

type QueryOptions struct {
	// 关闭查询
	disable bool

	// 查询请求前回调
	beforeBuilderCallback RequestBuilderCallback

	// 查询请求后回调
	afterBuilderCallback RequestBuilderCallback

	// responseSuccessCallback 成功响应回调
	responseSuccessCallback gateway.ResponseSuccessCallback

	// 查询中间件
	middlewares []gateway.Handler
}

type GetByIDOptions struct {
	// 关闭根据ID查询
	disable bool

	// 根据ID请求前回调
	beforeBuilderCallback RequestBuilderCallback

	// 根据ID请求后回调
	afterBuilderCallback RequestBuilderCallback

	// responseSuccessCallback 成功响应回调
	responseSuccessCallback gateway.ResponseSuccessCallback

	// 根据ID查询中间件
	middlewares []gateway.Handler
}

func WithGlobalMiddlewares(middlewares ...gateway.Handler) GlobalOption {
	return func(options *GlobalOptions) {
		options.middlewares = middlewares
	}
}

func WithDisableCreate() CreateOption {
	return func(options *CreateOptions) {
		options.disable = true
	}
}

func WithBeforeCreateBuilderCallback(callbacks RequestBuilderCallback) CreateOption {
	return func(options *CreateOptions) {
		options.beforeBuilderCallback = callbacks
	}
}

func WithAfterCreateBuilderCallback(callbacks RequestBuilderCallback) CreateOption {
	return func(options *CreateOptions) {
		options.afterBuilderCallback = callbacks
	}
}

func WithCreateResponseSuccessCallback(responseSuccessCallback gateway.ResponseSuccessCallback) CreateOption {
	return func(options *CreateOptions) {
		options.responseSuccessCallback = responseSuccessCallback
	}
}

func WithCreateMiddlewares(middlewares ...gateway.Handler) CreateOption {
	return func(options *CreateOptions) {
		options.middlewares = middlewares
	}
}

func WithDisableDelete() DeleteOption {
	return func(options *DeleteOptions) {
		options.disable = true
	}
}

func WithBeforeDeleteBuilderCallback(callbacks RequestBuilderCallback) DeleteOption {
	return func(options *DeleteOptions) {
		options.beforeBuilderCallback = callbacks
	}
}

func WithAfterDeleteBuilderCallback(callbacks RequestBuilderCallback) DeleteOption {
	return func(options *DeleteOptions) {
		options.afterBuilderCallback = callbacks
	}
}

func WithDeleteResponseSuccessCallback(responseSuccessCallback gateway.ResponseSuccessCallback) DeleteOption {
	return func(options *DeleteOptions) {
		options.responseSuccessCallback = responseSuccessCallback
	}
}

func WithDeleteMiddlewares(middlewares ...gateway.Handler) DeleteOption {
	return func(options *DeleteOptions) {
		options.middlewares = middlewares
	}
}

func WithDisableUpdate() UpdateOption {
	return func(options *UpdateOptions) {
		options.disable = true
	}
}

func WithBeforeUpdateBuilderCallback(callbacks RequestBuilderCallback) UpdateOption {
	return func(options *UpdateOptions) {
		options.beforeBuilderCallback = callbacks
	}
}

func WithAfterUpdateBuilderCallback(callbacks RequestBuilderCallback) UpdateOption {
	return func(options *UpdateOptions) {
		options.afterBuilderCallback = callbacks
	}
}

func WithUpdateResponseSuccessCallback(responseSuccessCallback gateway.ResponseSuccessCallback) UpdateOption {
	return func(options *UpdateOptions) {
		options.responseSuccessCallback = responseSuccessCallback
	}
}

func WithUpdateMiddlewares(middlewares ...gateway.Handler) UpdateOption {
	return func(options *UpdateOptions) {
		options.middlewares = middlewares
	}
}

func WithDisableQuery() QueryOption {
	return func(options *QueryOptions) {
		options.disable = true
	}
}

func WithBeforeQueryBuilderCallback(callbacks RequestBuilderCallback) QueryOption {
	return func(options *QueryOptions) {
		options.beforeBuilderCallback = callbacks
	}
}

func WithAfterQueryBuilderCallback(callbacks RequestBuilderCallback) QueryOption {
	return func(options *QueryOptions) {
		options.afterBuilderCallback = callbacks
	}
}

func WithQueryResponseSuccessCallback(responseSuccessCallback gateway.ResponseSuccessCallback) QueryOption {
	return func(options *QueryOptions) {
		options.responseSuccessCallback = responseSuccessCallback
	}
}

func WithQueryMiddlewares(middlewares ...gateway.Handler) QueryOption {
	return func(options *QueryOptions) {
		options.middlewares = middlewares
	}
}

func WithDisableGetByID() GetByIDOption {
	return func(options *GetByIDOptions) {
		options.disable = true
	}
}

func WithBeforeGetByIDBuilderCallback(callbacks RequestBuilderCallback) GetByIDOption {
	return func(options *GetByIDOptions) {
		options.beforeBuilderCallback = callbacks
	}
}

func WithAfterGetByIDBuilderCallback(callbacks RequestBuilderCallback) GetByIDOption {
	return func(options *GetByIDOptions) {
		options.afterBuilderCallback = callbacks
	}
}

func WithGetByIDResponseSuccessCallback(responseSuccessCallback gateway.ResponseSuccessCallback) GetByIDOption {
	return func(options *GetByIDOptions) {
		options.responseSuccessCallback = responseSuccessCallback
	}
}

func WithGetByIDMiddlewares(middlewares ...gateway.Handler) GetByIDOption {
	return func(options *GetByIDOptions) {
		options.middlewares = middlewares
	}
}
